package com.bwie.auth.service.impl;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bwie.auth.service.AuthService;
import com.bwie.common.constant.JwtConstants;
import com.bwie.common.constant.TokenConstants;

import com.bwie.common.domain.User;
import com.bwie.common.domain.request.LoginRequest;
import com.bwie.common.domain.request.PhoneLoginRequest;
import com.bwie.common.domain.request.UserReq;
import com.bwie.common.domain.request.UserRequest;
import com.bwie.common.domain.response.system.UserResponse;
import com.bwie.common.redis.RedisCache;
import com.bwie.common.remote.RemoteHouseService;
import com.bwie.common.remote.RemoteLoginService;
import com.bwie.common.result.BizException;
import com.bwie.common.result.Result;
import com.bwie.common.utils.IdUtils;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.StringUtils;
import io.jsonwebtoken.Claims;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

@Service
public class AuthServiceImpl implements AuthService {

    @Autowired
    private RemoteLoginService remoteLoginService;

    @Autowired
    private StringRedisTemplate redisTemplate;

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private HttpServletRequest request;


    @Override
    public UserResponse login(LoginRequest loginRequest) {

        Assert.isTrue(
                !StringUtils.isAnyBlank(loginRequest.getUserPwd(),
                loginRequest.getUserName()),"账号或密码不能为空"
        );
        Result<User> result = remoteLoginService.login(loginRequest);
        User user = result.getData();
        Assert.notNull(user,"该账号不是系统用户");

        HashMap<String, Object> map = new HashMap<>();
        String userKey = UUID.randomUUID().toString().replaceAll("-", "");
        map.put(JwtConstants.USER_KEY,userKey);
        map.put(JwtConstants.DETAILS_USERNAME,user.getUserName());
        map.put(JwtConstants.DETAILS_USER_ID,user.getUserId());
        String token = JwtUtils.createToken(map);
        redisCache.setCacheObject(
                TokenConstants.LOGIN_TOKEN_KEY+userKey,
                JSON.toJSONString(user),
                TokenConstants.EXPIRATION,
                TimeUnit.SECONDS
        );
        return UserResponse.builder()
                .token(token)
                .expired(TokenConstants.EXPIRATION)
                .build();
    }

    @Override
    public Result reg(UserReq userReq) {
        String password = SecureUtil.md5(userReq.getUserPassword() + "|" +"");
        Result result = remoteLoginService.register(
                User.builder()
                        .userName(userReq.getUserName())
                        .userPassword(password)
                        .userTel(userReq.getUserTel())
                        .userStatus(userReq.getUserStatus())
                        .userRole(userReq.getUserRole())
                        .userStatus(userReq.getUserStatus())
                        .userImg(userReq.getUserImg())
                        .build()
        );
        return result;
    }

    @Override
    public Result sendCode(String userTel) {
        Result<User> result = remoteLoginService.findByPhone(userTel);
        User user = result.getData();
        if(null == user){
            throw new BizException(408,"请先注册手机号");
        }
        String code = RandomUtil.randomNumbers(4);
        redisCache.setCacheObject("Send_sms_"+userTel,code);

        return Result.success("验证码为："+code,"发送成功");
    }


    @Override
    public Result sencCode(String userTel) {
        Result<User> result = remoteLoginService.phoneLogin(userTel);
        if(StringUtils.isNull(result.getData())){
            return Result.error("手机号不存在");
        }
        String code = RandomUtil.randomNumbers(4);
        System.out.println("验证码为："+code);
        redisTemplate.opsForValue().set(userTel,code,5,TimeUnit.MINUTES);

        return result;
    }

    @Override
    public Result phoneLogin(PhoneLoginRequest phoneLoginRequest) {
        Result<User> result = remoteLoginService.phoneLogin(phoneLoginRequest.getUserTel());
        User user = result.getData();
        if(null==user){
            return Result.error("手机号不存在");
        }
        if(!redisTemplate.hasKey(phoneLoginRequest.getUserTel())){
            return Result.error("验证码过期");
        }
        String code = redisTemplate.opsForValue().get(phoneLoginRequest.getUserTel());
        if(!phoneLoginRequest.getUserCode().equals(code)){
            return Result.error("验证码错误");
        }
        HashMap<String, Object> map = new HashMap<>();
        String userKey = UUID.randomUUID().toString().replaceAll("-","");
        map.put(JwtConstants.USER_KEY,userKey);
        String token = JwtUtils.createToken(map);
        redisCache.setCacheObject(TokenConstants.TOKEN+token, user, TokenConstants.EXPIRATION, TimeUnit.HOURS);

        UserResponse userResponse = new UserResponse();
        userResponse.setToken(token);
        userResponse.setExpired(TokenConstants.EXPIRATION);
        return Result.success(userResponse,"登陆成功");
    }

    @Override
    public User getInfo() {
        String token = request.getHeader(TokenConstants.TOKEN);
        Claims claims = JwtUtils.parseToken(token);
        String userKey = JwtUtils.getUserKey(claims);
        String cacheObject = redisCache.getCacheObject(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        return JSONObject.parseObject(cacheObject, User.class);
    }

}
